#include "mex.h"
#include "multi.h"
#include <numeric>
#include <time.h>
#include <curand.h>
#include <curand_kernel.h>
#include <stdio.h>
#include "matrix.h"



//using namespace std;
 

 
 
//	multi(pr_c, row_idx, col_idx, psi, theta, all_i, nnz, x_kn, x_mk); 
void multi(int const *pr_c, int const *row_idx, int const *col_idx, const Matrix psi, const Matrix theta, const int all, const int nnz, Matrix_i x_kn, Matrix_i x_mk) {
	
	



	
	//init CUDA
	cudaSetDevice(0);
	cudaDeviceSynchronize();

	
	
	
	

	
	
	
	
	
	// Jc, Ir -> GPU

	//col_idx
	int* d_col_idx;
	size_t size_col_idx = nnz * sizeof(int);
	cudaMalloc(&d_col_idx, size_col_idx);
	cudaMemcpy(d_col_idx, col_idx, size_col_idx, cudaMemcpyHostToDevice);
	
	
	//row_idx
	int* d_row_idx;
	size_t size_row_idx = nnz * sizeof(int);
	cudaMalloc(&d_row_idx, size_row_idx);
	cudaMemcpy(d_row_idx, row_idx, size_row_idx, cudaMemcpyHostToDevice);
	

	

	// Load psi to device memory
	Matrix d_psi;
	size_t size_psi = psi.width * psi.height * sizeof(float);
	cudaMalloc(&d_psi.elements, size_psi);
	cudaMemcpy(d_psi.elements, psi.elements, size_psi, cudaMemcpyHostToDevice);
		
	// Load theta to device memory
	Matrix d_theta;
	size_t size_theta = theta.width * theta.height * sizeof(float);
	cudaMalloc(&d_theta.elements, size_theta);
	cudaMemcpy(d_theta.elements, theta.elements, size_theta, cudaMemcpyHostToDevice);
	
	
	
	// Allocate x_kn in device memory
	Matrix_i d_x_kn;
	size_t size_x_kn = all * sizeof(int);
	cudaMalloc(&d_x_kn.elements, size_x_kn);
	
	
	// Allocate x_mk in device memory
	Matrix_i d_x_mk;
	size_t size_x_mk = all * sizeof(int);
	cudaMalloc(&d_x_mk.elements, size_x_mk);

	
	
	
	
	
	Matrix test;
	test.height = 1;
	test.width = all;
	test.elements = (float*)malloc(all * sizeof(float));
	
	
		// Allocate x_kn in device memory
	Matrix d_test;
	size_t size_test = all * sizeof(float);
	cudaMalloc(&d_test.elements, size_test);
	
	
	
	
	
	Matrix_i index;
	index.height = all;
	index.width = 1;
	index.elements = (int*)malloc(index.width * index.height * sizeof(int));
	int tmp_idx = 0;
	for(int i = 0;i < nnz;i++){
		for(int j = 0;j < pr_c[i];j++){
			index.elements[tmp_idx] = i;
			tmp_idx++;
		}
	}
	


	
	
		// Allocate index in device memory
	Matrix_i d_index;
	d_index.width = index.width;
	d_index.height = index.height;
	size_t size_index = index.width * index.height * sizeof(float);
	cudaMalloc(&d_index.elements, size_index);
	cudaMemcpy(d_index.elements, index.elements, size_index, cudaMemcpyHostToDevice);
		
		
		

	
	
	
	
	
	
	// a table for perem
	Matrix_i para;
	para.height = 5;
	para.width = 1;
	para.elements = (int*)malloc(para.width * para.height * sizeof(int));
	para.elements[0] = psi.height; //M
	para.elements[1] = theta.width;//N
	para.elements[2] = psi.width;//K
	para.elements[3] = all;
	para.elements[4] = nnz;
	
	// Allocate para in device memory
	Matrix_i d_para;
	size_t size_para = para.width * para.height * sizeof(int);
	cudaMalloc(&d_para.elements, size_para);
	cudaMemcpy(d_para.elements, para.elements, size_para, cudaMemcpyHostToDevice);
	
	
	
	
	// Invoke kernel
	const int BlOCK_SIZE = 32;
	const dim3 dimBlock(BlOCK_SIZE);
	int tmp_grid = (all + BlOCK_SIZE - 1)/BlOCK_SIZE;
	const dim3 dimGrid(tmp_grid);
	

	
		
		//start clock 
   // clock_t start_time=clock();

	
	
	//half of the time is spent in kernal
	multi_kernal<<<dimGrid, dimBlock>>>(d_row_idx, d_col_idx, d_psi, d_theta, d_x_kn, d_x_mk, d_para, d_index, d_test);
	cudaDeviceSynchronize();
		
			//end clock
	//clock_t end_time=clock();
	//float clock = static_cast<float>(end_time-start_time)/CLOCKS_PER_SEC*1000;
	//printf("kernal time is: %lf ms.\n", clock);
	
	
	
	cudaMemcpy(x_kn.elements, d_x_kn.elements, size_x_kn, cudaMemcpyDeviceToHost);
	cudaMemcpy(x_mk.elements, d_x_mk.elements, size_x_mk, cudaMemcpyDeviceToHost);
	cudaMemcpy(test.elements, d_test.elements, size_test, cudaMemcpyDeviceToHost);
	

	//printf("test.elements[0] is %lf\n", test.elements[0]);
	
	
	
	// Free device memory
	cudaFree(d_psi.elements);
	cudaFree(d_theta.elements);
	cudaFree(d_x_kn.elements);
	cudaFree(d_x_mk.elements);
	cudaFree(d_para.elements);
	cudaFree(d_index.elements); 
	cudaFree(d_row_idx);
	cudaFree(d_col_idx);
	cudaFree(d_test.elements);
	free(para.elements);
	free(test.elements);
	free(index.elements);


	
	

}



__global__ void multi_kernal(int *row_idx, int *col_idx, Matrix psi, Matrix theta, Matrix_i x_kn, Matrix_i x_mk, Matrix_i para, Matrix_i index, Matrix test) {
	// Each thread sample once

	
	int idx = blockIdx.x * blockDim.x + threadIdx.x;

	//int p = para.elements[0];
	int N = para.elements[1];
	int k = para.elements[2];
	if(idx > para.elements[3] - 1) return;

	

	//know which element of xpn is in the kernel, i.e the idx in the pr.   ONLY BOTTLENECK of the KERNAL!!!!!!
	int x_idx = index.elements[idx];
	

	//row and col
	int row = row_idx[x_idx];
	int col = col_idx[x_idx];
	
 	//random 
	curandState state;
	curand_init(clock64(), 0, 0, &state);  
	


	//calculate the row of psi times col of theta
	float pp = 0;
	int psi_init = k * row;
	int theta_init = col;
	//k-1 or k?
	for(int i=0;i < k;i++) {
		pp += psi.elements[i+psi_init] * theta.elements[i*N+theta_init];
	}

	float da = psi.elements[psi_init] * theta.elements[theta_init];
	float rr = curand_uniform(&state)*pp;
	
	
	int out = 0; 		
	//here n < k-1 is because n = k-1 cannot make out++, HOWEVER in some cases it DOES and cause out==k, which crashed matlab 
	for (int n = 0; n < k-1; n++) {
		if(n>0){
			da += psi.elements[row*k + n] * theta.elements[col + n*N];
		}
		
	
		if(da > rr) {
			break;
		}

		out++;
		
	}

	
	
	
	
	x_kn.elements[idx] = ((out*N) + col);  
	x_mk.elements[idx] = ((row*k) + out);   

}




//[xmk_new, xkn_new] = test_sparse(xpn_single_ceil, s_psi, s_theta, summ)  <- MATLAB
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{


	//define
	Matrix psi, theta, xmk, xkn;
	Matrix_i x_kn, x_mk;
	float all;
	all = mxGetScalar(prhs[3]);
	int all_i = (int)(all);
	
	// sizes
	mwSize M = mxGetM( prhs[1] );
	mwSize N = mxGetN( prhs[2] );
	mwSize K = mxGetN( prhs[1] );
	
	//data of xpn
	
	double *pr = mxGetPr( prhs[0] );
	mwIndex *jc = mxGetJc( prhs[0] );
	mwIndex *ir = mxGetIr( prhs[0] );
	

	
	//calculate the nnz: number of non-zero

	
	int nnz = jc[N];
	
	// calculate cumulative Pr, i.e. Pr_c
	int* pr_c;
	pr_c = (int*)malloc(nnz * sizeof(int));
	for(int i = 0; i<nnz; i++){
		pr_c[i] = int(pr[i]);
	}
	
	// transform ir to int*  
	int* row_idx;
	row_idx = (int*)malloc(nnz * sizeof(int));
	for(int i = 0; i<nnz; i++){
		row_idx[i] = ir[i];
	}
	

	
	
	//calculate the col index from jc, correct
	int* col_idx;
	col_idx = (int*)malloc(nnz * sizeof(int));
	int tmp = 0;
	for(int i = 0; i<N; i++){
		int nnz_col_i = jc[i+1] - jc[i];
		if(nnz_col_i > 0) {
			for(int ii=0;ii<nnz_col_i;ii++){
				col_idx[tmp] = i;
				tmp++;
			}
		}
	}
	
	
	//data of psi
	float* v_psi = (float*)(mxGetData(prhs[1]));
	psi.height = (int)(M);
	psi.width = (int)(K);
	psi.elements = (float*)malloc(psi.width * psi.height * sizeof(float));
	for(int i = 0; i < psi.height; i++){
		for(int j = 0; j < psi.width; j++){
			psi.elements[i*psi.width + j] = v_psi[i + j*psi.height];
		}
	}

	
	//data of theta
	float* v_theta = (float*)(mxGetData(prhs[2]));
	theta.height = (int)(K);
	theta.width = (int)(N);
	theta.elements = (float*)malloc(theta.width * theta.height * sizeof(float));
	for(int i = 0; i < theta.height; i++){
		for(int j = 0; j < theta.width; j++){
			theta.elements[i*theta.width + j] = v_theta[i + j*theta.height];
		}
	}

	

	//allocate space for x_kn and x_mk
	x_kn.elements = (int*)malloc(all_i * sizeof(int));
	x_mk.elements = (int*)malloc(all_i * sizeof(int));



	


	//multi is the ONLY BOTTLENECK!

	multi(pr_c, row_idx, col_idx, psi, theta, all_i, nnz, x_kn, x_mk); 
	


	
	
		//xmk
	xmk.height = (int)(M);
	xmk.width = (int)(K);
	xmk.elements = (float*)malloc(xmk.width * xmk.height * sizeof(float));
	for(int i = 0; i < M*K; i++){
		xmk.elements[i] = 0;
	}
	//xkn
	xkn.height = (int)(K);
	xkn.width = (int)(N);
	xkn.elements = (float*)malloc(xkn.width * xkn.height * sizeof(float));
	for(int i = 0; i < K*N; i++){
		xkn.elements[i] = 0;
	}
	

	for(int a=0;a<all_i;a++){
		xkn.elements[x_kn.elements[a]]++;
		xmk.elements[x_mk.elements[a]]++; 
	}


	//push xmk to plhs
	plhs[0] = mxCreateNumericMatrix(M, K, mxSINGLE_CLASS, mxREAL);
	float* y;
	y = (float*)mxGetData(plhs[0]);
	int ny = (int)(mxGetM(plhs[0]));
	int yi,yj;
	for(yi=0;yi<xmk.height;yi++)
        for(yj=0;yj<xmk.width;yj++)
            *(y+yj*ny+yi)=xmk.elements[yi*(0 + xmk.width) + yj];
	//push xkn to plhs
	plhs[1] = mxCreateNumericMatrix(K, N, mxSINGLE_CLASS, mxREAL);
	float* z;
	z = (float*)mxGetData(plhs[1]);
	int nz = (int)(mxGetM(plhs[1]));
	int zi,zj;
	for(zi=0;zi<xkn.height;zi++)
        for(zj=0;zj<xkn.width;zj++)
            *(z+zj*nz+zi)=xkn.elements[zi*(0 + xkn.width) + zj];
	

	
	
	//free
	free(psi.elements);
	free(theta.elements);
	free(x_kn.elements);
	free(x_mk.elements);
	free(xkn.elements);
	free(xmk.elements);
	
	

}